#include <QMainWindow>
#include <QApplication>
#include <QVBoxLayout>
#include <QLineEdit>
#include <QStatusBar>
#include <QMenuBar>
#include <QMenu>
#include <QAction>
#include <QGroupBox>
#include <QQuickWidget>
#include <QQmlError>
#include <QQuickView>

#include <QtDebug>

static void dumpWindowRecursion(QTextStream &str, const QWindow *w, const QWindowList &allWins, int depth = 0)
{
    if (depth)
        str << QString(depth * 2, QLatin1Char(' '));
    const QRect geom = w->geometry();
    const QPoint globalPos = w->isTopLevel() ? geom.topLeft() : w->mapToGlobal(geom.topLeft());
    str << '"' << w->metaObject()->className() << "\"/\"" << w->objectName() << "\" "
        << (w->isVisible() ? "[visible] " : "[hidden] ");
    if (w->isTopLevel())
        str << "[top] ";
    if (w->surfaceType() == QSurface::OpenGLSurface)
        str << "[gl] ";
     const QSurfaceFormat format = w->requestedFormat();
     if (format.hasAlpha())
         str << "[alpha] ";
     str << "f=0x" << hex << unsigned(w->flags()) << dec
        << "\" " << geom.width() << 'x' << geom.height()
        << forcesign << geom.x() << geom.y()
        << " Global: " << globalPos.x() << globalPos.y() << noforcesign << '\n';
    foreach (const QWindow *cw, allWins)
        if (cw->type() != Qt::Desktop && cw->parent() == w)
            dumpWindowRecursion(str, cw, allWins, depth + 1);
}
static void dumpAllWindows()
{
    QString d;
    QWindowList wins = QGuiApplication::allWindows();
    QTextStream str(&d);
    foreach (const QWindow *w, wins)
        if (w->type() != Qt::Desktop && w->isTopLevel())
            dumpWindowRecursion(str, w, wins, 0);
    qDebug().nospace() << d;
}

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    MainWindow();

public slots:
    void dump();
};

void MainWindow::dump()
{
    qDebug() << "focus" << QGuiApplication::focusWindow();
    dumpAllWindows();
}

MainWindow::MainWindow()
{
    setWindowTitle(QT_VERSION_STR);
    QWidget *centralWidget = new QWidget(this);
    QVBoxLayout *layout = new QVBoxLayout(centralWidget);
    QStringList args = QCoreApplication::arguments();

    QLineEdit *le = new QLineEdit();
    le->setObjectName(QLatin1String("lineedit1"));
    layout->addWidget(le);

    const QUrl u = QUrl(QStringLiteral("qrc:///embeddedinwidgets/main.qml"));

    if (!args.contains("-no-wc")) {
        QQuickView *quickView = new QQuickView(u);
        quickView->setResizeMode(QQuickView::SizeRootObjectToView);
        QWidget *container = QWidget::createWindowContainer(quickView);
        container->setMinimumSize(quickView->size());
        QGroupBox *g = new QGroupBox("WindowContainer");
        QVBoxLayout *gbl = new QVBoxLayout(g);
        gbl->addWidget(container);
        layout->addWidget(g);
    }

    if (!args.contains("-no-qw")) {
        QGroupBox *g = new QGroupBox("QuickWidget");
        QVBoxLayout *gbl = new QVBoxLayout(g);
        QQuickWidget *qw = new QQuickWidget(u, g);
        qw->setFocusPolicy(Qt::ClickFocus);
        gbl->addWidget(qw);
        layout->addWidget(g);
    }

    setCentralWidget(centralWidget);

    QMenu *fileMenu = menuBar()->addMenu("File");
    QAction *dumpAction = fileMenu->addAction(tr("Dump"));
    dumpAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_D));
    connect(dumpAction, SIGNAL(triggered()), this, SLOT(dump()));

    QAction *quitAction = fileMenu->addAction(tr("Quit"));
    quitAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q));
    connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
}

int main(int argc, char **argv)
{
    QApplication app(argc, argv);

    MainWindow mainWindow;
    mainWindow.move(300, 300);
    mainWindow.show();

    return app.exec();
}

#include "main.moc"
